/* SPDX-License-Identifier: BSD-2-Clause */
/***********************************************************************;
 * Copyright (c) 2015 - 2017, Intel Corporation
 * All rights reserved.
 ***********************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h" // IWYU pragma: keep
#endif

#include "sysapi_util.h"     // for _TSS2_SYS_CONTEXT_BLOB, syscontext_cast
#include "tss2_common.h"     // for TSS2_RC, TSS2_SYS_RC_BAD_REFERENCE
#include "tss2_mu.h"         // for Tss2_MU_UINT16_Marshal, Tss2_MU_TPM2B_A...
#include "tss2_sys.h"        // for TSS2_SYS_CONTEXT, TSS2L_SYS_AUTH_COMMAND
#include "tss2_tpm2_types.h" // for TPM2B_AUTH, TPM2B_NV_PUBLIC, TPMI_RH_PR...

TSS2_RC
Tss2_Sys_NV_DefineSpace_Prepare(TSS2_SYS_CONTEXT      *sysContext,
                                TPMI_RH_PROVISION      authHandle,
                                const TPM2B_AUTH      *auth,
                                const TPM2B_NV_PUBLIC *publicInfo) {
    TSS2_SYS_CONTEXT_BLOB *ctx = syscontext_cast(sysContext);
    TSS2_RC                rval;

    if (!ctx)
        return TSS2_SYS_RC_BAD_REFERENCE;

    rval = ValidateNV_Public(publicInfo);
    if (rval)
        return rval;

    rval = CommonPreparePrologue(ctx, TPM2_CC_NV_DefineSpace);
    if (rval)
        return rval;

    rval = Tss2_MU_UINT32_Marshal(authHandle, ctx->cmdBuffer, ctx->maxCmdSize, &ctx->nextData);
    if (rval)
        return rval;

    if (!auth) {
        ctx->decryptNull = 1;

        rval = Tss2_MU_UINT16_Marshal(0, ctx->cmdBuffer, ctx->maxCmdSize, &ctx->nextData);
    } else {

        rval = Tss2_MU_TPM2B_AUTH_Marshal(auth, ctx->cmdBuffer, ctx->maxCmdSize, &ctx->nextData);
    }

    if (rval)
        return rval;

    if (!publicInfo) {
        rval = Tss2_MU_UINT16_Marshal(0, ctx->cmdBuffer, ctx->maxCmdSize, &ctx->nextData);

    } else {

        rval = Tss2_MU_TPM2B_NV_PUBLIC_Marshal(publicInfo, ctx->cmdBuffer, ctx->maxCmdSize,
                                               &ctx->nextData);
    }

    if (rval)
        return rval;

    ctx->decryptAllowed = 1;
    ctx->encryptAllowed = 0;
    ctx->authAllowed = 1;

    return CommonPrepareEpilogue(ctx);
}

TSS2_RC
Tss2_Sys_NV_DefineSpace_Complete(TSS2_SYS_CONTEXT *sysContext) {
    TSS2_SYS_CONTEXT_BLOB *ctx = syscontext_cast(sysContext);

    if (!ctx)
        return TSS2_SYS_RC_BAD_REFERENCE;

    return CommonComplete(ctx);
}

TSS2_RC
Tss2_Sys_NV_DefineSpace(TSS2_SYS_CONTEXT             *sysContext,
                        TPMI_RH_PROVISION             authHandle,
                        TSS2L_SYS_AUTH_COMMAND const *cmdAuthsArray,
                        const TPM2B_AUTH             *auth,
                        const TPM2B_NV_PUBLIC        *publicInfo,
                        TSS2L_SYS_AUTH_RESPONSE      *rspAuthsArray) {
    TSS2_SYS_CONTEXT_BLOB *ctx = syscontext_cast(sysContext);
    TSS2_RC                rval;

    rval = Tss2_Sys_NV_DefineSpace_Prepare(sysContext, authHandle, auth, publicInfo);
    if (rval)
        return rval;

    rval = CommonOneCall(ctx, cmdAuthsArray, rspAuthsArray);
    if (rval)
        return rval;

    return Tss2_Sys_NV_DefineSpace_Complete(sysContext);
}
